/*
    Trabalho Prtico I - IPDI
    -------------------------
    Anderson de Rezende Rocha <anderson.rocha@ic.unicamp.br>
    Leyza E. Baldo Dorini <leyzad@yahoo.com.br>

    Implementacao do filtro do calculo do erro mdio quadrtico utilizando uma mscara de ativao

    OBSERVAO
    ----------
    Imagem[i,j] = *(Imagem + i * nc + j)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "imagem.h"
#include "morfo.h"
#include "utils.h"
#include "tratadorErros.h"

int calculaErro(int *imagemOriginal, int *imagemProcessada, int *imagemBinaria, int nl, int nc, int tamanhoJanela);

static int TAMJANELA;

int main (int argc, char *argv[])
{
    int OK, nc, nl, mn;
    int *imagemEntrada;
    int *imagemSaida;
    int *imagemAtivacao;
    int l,  m;

    printf("\nClculo do RMS (Rate Mean Square) by <Anderson Rocha, Leyza Dorini>\n");
    if (argc == 5)
    {
        OK = le_imagem_pgm (argv[1], &imagemEntrada, &nl, &nc, &mn);
        OK = le_imagem_pgm (argv[2], &imagemSaida, &nl, &nc, &mn);
        OK = le_imagem_pgm (argv[3], &imagemAtivacao, &nl, &nc, &mn);
        TAMJANELA = atoi(argv[4]);        

        info_imagem (argv[1], nl, nc, mn);
        info_imagem (argv[2], nl, nc, mn);       
        info_imagem (argv[3], nl, nc, mn);

        printf("RMS de \"%s\" em relacao \"%s\" = %d, utiliando como mascara de ativacao \"%s\"\n", argv[1], argv[2],
               calculaErro(imagemEntrada, imagemSaida, imagemAtivacao, nl, nc, TAMJANELA), argv[3]);

        desaloca_memo(&imagemSaida);
    }
    else
    {
        tratarSintaxe(5);
    }

    printf("\n");
    return(0);
}

int calculaErro(int *imagemOriginal, int *imagemProcessada, int *imagemBinaria, int nl, int nc, int tamanhoJanela)
{
    int i,  j;
    int soma, aux, aux2, auxiliar, erro, erroFinal;
    // Linha inicial e coluna inicial
    // Linha final e coluna final
    int li, ci, lf, cf;

    li = tamanhoJanela/2;
    ci = tamanhoJanela/2;
    lf = nl - li;
    cf = nc - ci;

    soma = 0;
    erro = 0;
    erroFinal = 0;
    aux  = 0;
    aux2  = 0;   
    auxiliar = 0;

    for(i = li; i < lf; i++)
    {
        for(j = ci; j < cf; j++)
        {
            //utiliza a imagemBinaria como uma mscara de ativao do clculo do erro.
            //Sendo assim,  possvel calcular o RMS High, referente s regies de alta atividade espacial
            //e tambm o RMS Low correspondente s regies de baixa atividade espacial.
            //Estas mscaras binrias foram geradas no programa prewitt2.c
            auxiliar = *(imagemBinaria + i * nc + j);
            if(auxiliar == 255)
            {
                aux = *(imagemProcessada + i * nc + j) - *(imagemOriginal + i * nc + j);
                aux2 = pow(aux, 2);
                soma = soma + aux2;
            }
        }
    }

    erro = soma / (nl*nc);
    erroFinal = sqrt(erro);

    return erroFinal;
}